/**
 * @file OTA-mDNS-SPIFFS.ino
 * 
 * @author Pascal Gollor (http://www.pgollor.de/cms/)
 * @date 2015-09-18
 * 
 * changelog:
 * 2015-10-22: 
 * - Use new ArduinoOTA library.
 * - loadConfig function can handle different line endings
 * - remove mDNS studd. ArduinoOTA handle it.
 * 
 */

// includes
#include <ESP8266WiFi.h>
#include <ESP8266mDNS.h>
//#include <WiFiUdp.h>
#include <FS.h>
#include <ArduinoOTA.h>
//#include <DHT.h>
//#include <DHT_U.h>

#include <ESP8266WebServer.h>
ESP8266WebServer server ( 80 );

// RGB PINs
#define redPIN    15
#define greenPIN  13
#define bluePIN   12

// W1, W2 Pins
#define w1PIN     14
#define w2PIN     4

// onbaord green LED D1
#define LEDPIN    16      // für reale Schaltung Pin 1 verwenden
// onbaord red LED D2
#define LED2PIN   5


#define LEDoff digitalWrite(LEDPIN,HIGH)
#define LEDon digitalWrite(LEDPIN,LOW)

#define LED2off digitalWrite(LED2PIN,HIGH)
#define LED2on digitalWrite(LED2PIN,LOW)

int led_delay_red             = 0;
int led_delay_green           = 0;
int led_delay_blue            = 0;
int led_delay_w1              = 0;
int led_delay_w2              = 0;
//#define time_at_colour 1300 
//unsigned long TIME_LED_RED    = 0;
//unsigned long TIME_LED_GREEN  = 0;
//unsigned long TIME_LED_BLUE   = 0;
//unsigned long TIME_LED_W1     = 0;
//unsigned long TIME_LED_W2     = 0;
int t_slide                  = 1;   // Dämpfung mit der Farbwertänderungen durchgeführt werden.
long m_zeit                   = 0;       // Systemzeit an der die letzte Änderung der Farbwerte durchgeführt wurde oder an der ein neuer Farbwert gesetzt wurde
int RED, GREEN, BLUE, W1, W2;   // Zielfarbwerte
int RED_A = 0;        // aktueller Farbwert
int GREEN_A = 0;      // aktueller Farbwert
int BLUE_A = 0;       // aktueller Farbwert
int W1_A = 0;         // aktueller Farbwert
int W2_A = 0;         // aktueller Farbwert

bool con_AP = false;  // gibt an ob mit dem im der Konfig-Datei konfigurierten AccessPoint verbunden

/**
 * @brief mDNS and OTA Constants
 * @{
 */
String HOSTNAME = "not-set"; ///< Hostename. Wird später aus der Datei cl_name-txt gelesen.
/// @}

/**
 * @brief Default WiFi connection information.
 * @{
 */
const char* ap_default_ssid = "esp8266"; ///< Default SSID.
const char* ap_default_psk = "esp8266esp8266"; ///< Default PSK.
/// @}

String station_ssid = "FRITZ!Box 7490";
String station_psk = "xxx";

/// Uncomment the next line for verbose output over UART.
//#define SERIAL_VERBOSE

/**
 * @brief Read WiFi connection information from file system.
 * @param ssid String pointer for storing SSID.
 * @param pass String pointer for storing PSK.
 * @return True or False.
 * 
 * The config file have to containt the WiFi SSID in the first line
 * and the WiFi PSK in the second line.
 * Line seperator can be \r\n (CR LF) \r or \n.
 */

/**
 * @brief Arduino setup function.
 */
void setup()
{
  Serial.begin(115200);
  
  delay(100);

  Serial.println("\r\n");
  Serial.print("Chip ID: 0x");
  Serial.println(ESP.getChipId(), HEX);

  // Initialize file system.
  if (!SPIFFS.begin())
  {
    Serial.println("Failed to mount file system");
    return;
  }

  // Load Hostname connection information.
  String hostname_temp = "";
  if (! loadhostname(&HOSTNAME))
  {
    HOSTNAME = "not-set";

    Serial.println("No HOSTNAME information available.");
  }

  // Set Hostname
  String hostname(HOSTNAME);
  // hostname += String(ESP.getChipId(), HEX);
  WiFi.hostname(hostname);

  // Print hostname.
  Serial.println("Hostname: " + hostname);
  //Serial.println(WiFi.hostname());


  // Load wifi connection information.
  if (! loadConfig(&station_ssid, &station_psk))
  {
    station_ssid = "";
    station_psk = "";

    Serial.println("No WiFi connection information available.");
  }

  // Check WiFi connection
  // ... check mode
  if (WiFi.getMode() != WIFI_STA)
  {
    WiFi.mode(WIFI_STA);
    delay(10);
  }

  // ... Compare file config with sdk config.
  if (WiFi.SSID() != station_ssid || WiFi.psk() != station_psk)
  {
    Serial.println("WiFi config changed.");

    // ... Try to connect to WiFi station.
    WiFi.begin(station_ssid.c_str(), station_psk.c_str());

    // ... Pritn new SSID
    Serial.print("new SSID: ");
    Serial.println(WiFi.SSID());

    // ... Uncomment this for debugging output.
    //WiFi.printDiag(Serial);
  }
  else
  {
    // ... Begin with sdk config.
    WiFi.begin();
  }

  Serial.println("Wait for WiFi connection.");

  // ... Give ESP 10 seconds to connect to station.
  unsigned long startTime = millis();
  while (WiFi.status() != WL_CONNECTED && millis() - startTime < 10000)
  {
    Serial.write('.');
    //Serial.print(WiFi.status());
    delay(500);
  }
  Serial.println();

  // Check connection
  if(WiFi.status() == WL_CONNECTED)
  {
    // ... print IP Address
    Serial.print("IP address: ");
    Serial.println(WiFi.localIP());
    LED2on;
    con_AP = true;
  }
  else
  {
    Serial.println("Can not connect to WiFi station. Go into AP mode.");
    
    // Go into software AP mode.
    WiFi.mode(WIFI_AP);

    delay(10);

    WiFi.softAP(ap_default_ssid, ap_default_psk);

    Serial.print("IP address: ");
    Serial.println(WiFi.softAPIP());
    LED2off;
  }

  // Start OTA server.
  ArduinoOTA.setHostname((const char *)hostname.c_str());
  ArduinoOTA.begin();

// --------------------------------------------------------------
// --------- Code für Web-Server --------------------------------
// --------------------------------------------------------------
  server.on ( "/ledcontrol", ledcontrol );
  server.onNotFound ( handleNotFound );
  server.begin();
  server.on ( "/setdaempfung", setdaempfung );
  server.onNotFound ( handleNotFound );
  server.begin();
  server.on ( "/set", set );
  server.onNotFound ( handleNotFound );
  server.begin();
  server.on ( "/status", status );
  server.onNotFound ( handleNotFound );
  server.begin();
  server.on ( "/restart", restart );
  server.onNotFound ( handleNotFound );
  server.begin();
  server.on ( "/help", help );
  server.onNotFound ( handleNotFound );
  server.begin();
  Serial.println ( "HTTP server started" );

// --------------------------------------------------------------
// --------- LED Steuerung --------------------------------------
// --------------------------------------------------------------

  // Load Dämpfung.
  String dae = "";        // Variable für die Daempfung als String
  if (! loaddaempfung(&dae))
  {
    Serial.println("konnte Daempfung nicht laden.");
  }
  else          // wenn die Parameter gelesen werden konnten müssen sie noch in die globalen Variablen übertragen werden.
  {
        // Der String mit der Dämpfung muss umgewandelt werden
        // Der String mit dem Port muss umgewandelt werden
//        Serial.print("Dämpfung aus Datei = ");
//        Serial.println(dae);
        t_slide = (dae.toInt());
//        Serial.print("verwendete Dämpfung = ");
//        Serial.println(t_slide);
  }

  pinMode(LEDPIN, OUTPUT);  
  pinMode(LED2PIN, OUTPUT);  
  
  pinMode(redPIN, OUTPUT);
  pinMode(greenPIN, OUTPUT);
  pinMode(bluePIN, OUTPUT);
  pinMode(w1PIN, OUTPUT);
  pinMode(w2PIN, OUTPUT);    

  analogWriteRange(100);
}


/**
 * @brief Arduino loop function.
 */
void loop()
{
  // Handle OTA server.
  ArduinoOTA.handle();
  server.handleClient();
  yield();

  // Wenn das System nicht mit dem in der Konfig angegebenem Access-Point läuft wird das Netz gescannt.
  if (con_AP == false) {
    scan_connect(station_ssid);
  }

  // die Farbwerte werden langsam eingestellt
  if (millis() > m_zeit + t_slide) {            // wenn die Zeit seit der letzten Änderung vergangen ist...
    LED_RED();
    delay(1);
    LED_GREEN();
    delay(1);
    LED_BLUE();
    delay(1);
    LED_W1();
    delay(1);
    LED_W2();
    m_zeit = millis();                          // jetzt merken wir uns wieder die aktuelle Zeit
  }
}

void handleNotFound() {
  String message = "File Not Found\n\n";
  message += "URI: ";
  message += server.uri();
  message += "\nMethod: ";
  message += ( server.method() == HTTP_GET ) ? "GET" : "POST";
  message += "\nArguments: ";
  message += server.args();
  message += "\n";

  for ( uint8_t i = 0; i < server.args(); i++ ) {
    message += " " + server.argName ( i ) + ": " + server.arg ( i ) + "\n";
  }

  server.send ( 404, "text/plain", message );
}

